/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ws3dcontrol;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.logging.Level;
import java.util.logging.Logger;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.concurrent.Service;
import javafx.concurrent.Task;

/**
 *
 * @author jucah
 */
class Connection {

    protected Socket soc;
    protected OutputStream ostream;
    protected PrintWriter out;
    protected BufferedReader in;

    public Connection(String ip, int port) throws IOException {
        soc = new Socket(ip, port);
        ostream = soc.getOutputStream();
        out = new PrintWriter(ostream, true);
        in = new BufferedReader(new InputStreamReader(soc.getInputStream()));
    }
    
    public boolean isConnected() {
        if ((soc == null) || soc.isClosed()) {
            return false;
        } else {
            return true;
        }
    }
}

class SocketComm extends Task<String> {

    private final Connection _con;
    private final String _cmd;

    public SocketComm(Connection con, String cmd) {
        this._con = con;
        this._cmd = cmd;
    }

    @Override
    protected String call() throws Exception {
        if (_con != null) {
            _con.out.println(_cmd);
            _con.out.flush();
            StringBuilder resp = new StringBuilder();
            String line = null;
                while (!_con.in.ready());
                do {
                    line = _con.in.readLine();
                    if (line != null && !line.equals("")) {
                        //Logger.getLogger(WS3DProxy.class.getName()).log(Level.INFO, "Task[readline]: {0}", line);
                        resp.append(line).append("\r\n");
                    }
                    // else System.out.println("Error reading socket..."+st);

                }
                while (_con.in.ready());
            line = resp.toString();
            //Logger.getLogger(WS3DProxy.class.getName()).log(Level.INFO, "Task[response]: {0}", line);
            return line;
        } else {
            throw new Exception("No connection established!");
        }
    }
}

class WS3DProxy extends Service<String> {

    //private static Connection con = null;
    private static final String crlf = "\r\n";
    public ObjectProperty<Robot> robot = new SimpleObjectProperty();
    public ObjectProperty<Connection> con = new SimpleObjectProperty();
    public StringProperty cmd = new SimpleStringProperty();

    public final void setRobot(Robot r) {
        robot.set(r);
    }

    public final Robot getRobot() {
        return robot.get();
    }

    public final ObjectProperty robotProperty() {
        return robot;
    }

    public final void setCmd(String value) {
        cmd.set(value);
    }

    public final String getCmd() {
        return cmd.get();
    }

    public final StringProperty cmdProperty() {
        return cmd;
    }

    public final void setCon(Connection value) {
        con.set(value);
    }

    public final Connection getCon() {
        return con.get();
    }

    public final ObjectProperty<Connection> conProperty() {
        return con;
    }

    public WS3DProxy(String host, int port) throws IOException {
        this.connect(host, port);
    }

    @Override
    protected Task createTask() {
        final Connection _con = getCon();
        final String _cmd = getCmd();
        return new SocketComm(_con, _cmd);
    }
    
    @Override
    protected void succeeded() {
        super.succeeded();
        String lastCmd = this.cmdProperty().getValue();
        if (lastCmd.startsWith("stop")) {
            String response = this.getValue();
            String lastResponse = response.substring(response.indexOf("Coordinates"));
            String[] param = lastResponse.split("[\\s:]+");
            this.getRobot().setXCoord(Double.parseDouble(param[1]));
            this.getRobot().setYCoord(Double.parseDouble(param[2]));
            this.getRobot().setPitch(Double.parseDouble(param[3]));
        }
    }

    private void connect(String host, int port) throws IOException {
        try {
            if (getCon() != null && getCon().isConnected()) {
                StringBuilder resp = new StringBuilder();
                String line = null;
                while (!getCon().in.ready());
                do {
                    line = getCon().in.readLine();
                    if (line != null && !line.equals("")) {
                        //Logger.getLogger(WS3DProxy.class.getName()).log(Level.INFO, "connect[readline]: {0}", line);
                        resp.append(line).append(crlf);
                    }
                }
                while (getCon().in.ready());
                line = resp.toString();
                //Logger.getLogger(WS3DProxy.class.getName()).log(Level.INFO, "connect[response]: {0}", line);
                return;
            }
        } catch (Exception ex) {
        }
        setCon(new Connection(host, port));
        return;
    }
    
    public void disconnect() {
        try {
            getCon().soc.close();
        } catch (IOException ex) {
            Logger.getLogger(WS3DProxy.class.getName()).log(Level.SEVERE, null, ex);
        }
        System.exit(0);
    }
}
